@@ -10,11 +10,11 @@
     :copyright: 2006 by Armin Ronacher.
     :license: GNU GPL, see LICENSE for more details.
 """
-from pocoo.application import RequestHandler
+from pocoo.application import Page
 from pocoo.http import Response
 
 
-class CaptchaImage(RequestHandler):
+class CaptchaImage(Page):
     handler_regexes = ['!captcha$']
 
     def handle_request(self, req):
