@@ -12,11 +12,12 @@ from datetime import datetime
 from math import ceil
 
 from pocoo import Component
-from pocoo.db import meta, DatabaseModel, lazy_column
+from pocoo.db import meta, DatabaseModel, lazy_column, \
+     set_column_value, get_column_value, get_initial_column_value
 from pocoo.pkg.core.user import User
 from pocoo.pkg.core.db import forums, posts, users, ANONYMOUS_USER_ID
 from pocoo.pkg.core.template import Pagination, LazyPagination
-from pocoo.pkg.core.textfmt import parse_and_translate, quote_text
+from pocoo.pkg.core.textfmt import quote_text, parse, render, parse_and_render
 from pocoo.utils.uri import urlencode
 from pocoo.utils.iterators import inciter
 
@@ -84,13 +85,15 @@ def get_forum_index(req):
     categories = []
     def do(con):
         for category in con.execute(meta.select(columns,
-                                    f.parent_id == None)):
+                                    f.parent_id == None,
+                                    order_by=[meta.asc(f.position)])):
             category = dict(category)
             category['is_external_link'] = bool(category.pop('link'))
             category['url'] = ctx.make_url('forum', category['forum_id'])
             forums = []
             for forum in con.execute(meta.select(columns + [f.last_post_id],
-                                     f.parent_id == category['forum_id'])):
+                                     f.parent_id == category['forum_id'],
+                                     order_by=[meta.asc(f.position)])):
                 forum = dict(forum)
                 forum['is_external_link'] = bool(forum.pop('link'))
                 forum['url'] = ctx.make_url('forum', forum['forum_id'])
@@ -120,7 +123,8 @@ def get_forum_index(req):
                 forum['last_post'] = last_post
                 subforums = []
                 for sf in con.execute(meta.select([f.forum_id, f.name, f.link],
-                                      f.parent_id == forum['forum_id'])):
+                                      f.parent_id == forum['forum_id'],
+                                      order_by=[meta.asc(f.position)])):
                     sf = dict(sf)
                     sf['is_external_url'] = bool(sf['forum_id'])
                     sf['url'] = ctx.make_url('forum', sf.pop('forum_id'))
@@ -144,7 +148,7 @@ def get_forum(req, forum_id, page=1):
     p = posts.c
     u = users.c
     columns = [f.forum_id, f.description, f.name, f.link, f.post_count,
-               f.thread_count]
+               f.thread_count, f.local_thread_count]
     forum_columns = [f.forum_id, f.name, f.description, f.link,
                      f.post_count, f.thread_count, f.last_post_id]
     sf_columns = [f.forum_id, f.name, f.link]
@@ -152,8 +156,8 @@ def get_forum(req, forum_id, page=1):
                       p.post_count, p.view_count, p.username.label('guestname')]
     def do(con):
         category = con.execute(meta.select(columns,
-            f.forum_id == forum_id
-        )).fetchone()
+            f.forum_id == forum_id,
+        order_by=[meta.asc(f.position)])).fetchone()
         if category is None:
             return
         category = dict(category)
@@ -165,7 +169,8 @@ def get_forum(req, forum_id, page=1):
         category['is_external_url'] = bool(category['link'])
         forums = []
         for forum in con.execute(meta.select(forum_columns,
-                f.parent_id == category['forum_id'])):
+                f.parent_id == category['forum_id'],
+                order_by=[meta.asc(f.position)])):
             forum = dict(forum)
             forum['url'] = ctx.make_url('forum', forum['forum_id'])
             # wo don't pop that here so that the ForumPage request handler
@@ -175,7 +180,8 @@ def get_forum(req, forum_id, page=1):
             forum['is_external_link'] = bool(forum.pop('link'))
             subforums = []
             for sf in con.execute(meta.select(sf_columns,
-                    f.parent_id == forum['forum_id'])):
+                    f.parent_id == forum['forum_id'],
+                    order_by=[meta.asc(f.position)])):
                 sf = dict(sf)
                 sf['is_external_link'] = bool(sf.pop('link'))
                 sf['url'] = ctx.make_url('forum', sf['forum_id'])
@@ -216,7 +222,7 @@ def get_forum(req, forum_id, page=1):
                 link += '?page=%d' % number
             return link
         threads_per_page = get_threads_per_page(req)
-        page_count = int(ceil(category['thread_count'] / (threads_per_page * 1.0)))
+        page_count = int(ceil(category['local_thread_count'] / (threads_per_page * 1.0)))
         pagination = LazyPagination(req, page, threads_per_page, page_count,
                                     get_page_link)
 
@@ -327,8 +333,8 @@ def get_post_tree(req, post_id, inc_view_count=True):
 
     # load the post requested and lookup root_post_id
     result = ctx.engine.execute(meta.select([p.root_post_id, p.post_id, p.title,
-                                             p.text, p.timestamp, u.user_id,
-                                             u.register_date,
+                                             p.text, p.parsed_text, p.timestamp,
+                                             p.deleted, u.user_id, u.register_date,
                                              u.username, u.profile, u.post_count,
                                              p.username.label('guestname')],
         (p.post_id == post_id) &
@@ -336,7 +342,7 @@ def get_post_tree(req, post_id, inc_view_count=True):
     ))
     row = result.fetchone()
     if row is None:
-        # XXX: need error return here
+        # XXX: raise error here
         return
     post = dict(row)
     post['url'] = ctx.make_url('post', row['post_id'])
@@ -352,17 +358,22 @@ def get_post_tree(req, post_id, inc_view_count=True):
     }
     signature = None
     if post['author']['profile'].get('signature'):
-        signature = parse_and_translate(req, post['author']['profile']['signature'],
+        signature = parse_and_render(req, post['author']['profile']['signature'],
                                         signature=True)
     post['author']['signature'] = signature
 
-    #XXX: cache here
-    post['parsed_text'] = parse_and_translate(req, post['text'])
+    # if the post was deleted we remove the text from the return value
+    # so that templates don't display them by chance
+    if post['deleted']:
+        post['text'] = post['parsed_text'] = u''
+    else:
+        post['parsed_text'] = render(req, post['parsed_text'])
     root_post_id = post.pop('root_post_id')
 
     result = ctx.engine.execute(meta.select([p.post_id, p.root_post_id, p.title,
-                                             p.parent_id, p.timestamp, u.username,
-                                             u.user_id, p.username.label('guestname')],
+                                             p.deleted, p.parent_id, p.timestamp,
+                                             u.username, u.user_id,
+                                             p.username.label('guestname')],
         (p.root_post_id == root_post_id) &
         (u.user_id == p.author_id)
     ))
@@ -401,27 +412,24 @@ def get_post_tree(req, post_id, inc_view_count=True):
 
     # increment view_count
     if inc_view_count:
-        result = ctx.engine.execute(meta.select([p.view_count],
-                                    p.post_id == root_post_id))
-        ctx.engine.execute(posts.update(p.post_id == root_post_id),
-            view_count = result.fetchone()[0] + 1
-        )
-
+        ctx.engine.execute(posts.update(p.post_id == root_post_id,
+            values={'view_count': p.view_count + 1}
+        ))
 
     # fetch overall information for whole thread
-    result = ctx.engine.execute(meta.select([p.post_id, p.title, p.forum_id,
-                                             p.timestamp, u.user_id, u.username,
-                                             f.name, p.username.label('guestname')],
+    row = ctx.engine.execute(meta.select([p.post_id, p.title, p.forum_id, p.locked,
+                                          p.timestamp, u.user_id, u.username,
+                                          f.name, p.username.label('guestname')],
         (p.post_id == root_post_id) &
         (u.user_id == p.author_id) &
         (f.forum_id == p.forum_id)
-    ))
-    row = result.fetchone()
+    )).fetchone()
     return {
         'post_id':      row['post_id'],
         'url':          ctx.make_url('post', row['post_id']),
         'title':        row['title'],
         'timestamp':    row['timestamp'],
+        'locked':       row['locked'],
         'forum': {
             'forum_id':     row['forum_id'],
             'name':         row['name'],
@@ -447,14 +455,14 @@ def get_post(req, post_id):
     p = posts.c
     u = users.c
 
-    r = ctx.engine.execute(meta.select([p.post_id, p.title, p.text, p.timestamp,
-                                        u.user_id, u.username, u.profile,
-                                        u.register_date,
-                                        u.post_count, p.username.label('guestname')],
+    row = ctx.engine.execute(meta.select([p.post_id, p.title, p.text, p.deleted,
+                                          p.parsed_text, p.timestamp,
+                                          u.user_id, u.username, u.profile,
+                                          u.register_date, u.post_count,
+                                          p.username.label('guestname')],
         (p.post_id == post_id) &
         (u.user_id == p.author_id)
-    ))
-    row = r.fetchone()
+    )).fetchone()
     if row is None:
         return
     post = dict(row)
@@ -471,11 +479,15 @@ def get_post(req, post_id):
     }
     signature = None
     if post['author']['profile'].get('signature'):
-        signature = parse_and_translate(req, post['author']['profile']['signature'])
+        signature = parse_and_render(req, post['author']['profile']['signature'])
     post['author']['signature'] = signature
 
-    #XXX: cache here
-    post['parsed_text'] = parse_and_translate(req, post['text'])
+    # if the post was deleted we remove the text from the return value
+    # so that templates don't display them by chance
+    if post['deleted']:
+        post['text'] = post['parsed_text'] = u''
+    else:
+        post['parsed_text'] = render(req, post['parsed_text'])
     return post
 
 
@@ -486,9 +498,9 @@ def get_last_posts(req, root_post_id, n=1, offset=0):
     """
     p = posts.c
     u = users.c
-    result = req.ctx.engine.execute(meta.select([p.post_id, p.title, p.text,
-                                                 p.timestamp, u.username,
-                                                 u.user_id, u.profile,
+    result = req.ctx.engine.execute(meta.select([p.post_id, p.title, p.text, p.deleted,
+                                                 p.parsed_text, p.timestamp,
+                                                 u.username, u.user_id, u.profile,
                                                  u.post_count, u.register_date,
                                                  p.username.label('guestname')],
                 (p.root_post_id == root_post_id) &
@@ -514,14 +526,18 @@ def get_last_posts(req, root_post_id, n=1, offset=0):
         }
         signature = None
         if d['author']['profile'].get('signature'):
-            signature = parse_and_translate(req, d['author']['profile']['signature'],
+            signature = parse_and_render(req, d['author']['profile']['signature'],
                                             signature=True)
         d['author']['signature'] = signature
-        #XXX: this doesn't cache by now
-        d['parsed_text'] = parse_and_translate(req, d['text'])
+
+        # if the post was deleted we remove the text from the return value
+        # so that templates don't display them by chance
+        if d['deleted']:
+            d['text'] = d['parsed_text'] = u''
+        else:
+            d['parsed_text'] = render(req, d['parsed_text'])
         return d
-    post_list = [prepare(row) for row in result]
-    return post_list
+    return [prepare(row) for row in result]
 
 
 def get_flat_view(req, post_id, inc_view_count=True, order='asc'):
@@ -547,6 +563,7 @@ def get_flat_view(req, post_id, inc_view_count=True, order='asc'):
     result = ctx.engine.execute(meta.select([p.root_post_id],
         p.post_id == post_id
     ))
+
     # XXX: This raises TypeError on failure.
     root_post_id = result.fetchone()[0]
 
@@ -568,11 +585,13 @@ def get_flat_view(req, post_id, inc_view_count=True, order='asc'):
                             lambda x: 'post/%d' % x)
 
     orderfunc = (order == 'desc' and meta.desc or meta.asc)
+
     # select matching posts
     result = ctx.engine.execute(meta.select([p.post_id, p.root_post_id, p.title,
                                              p.forum_id, p.parent_id, p.text,
-                                             p.timestamp, u.username, u.user_id,
-                                             u.profile, u.post_count, u.register_date,
+                                             p.parsed_text, p.timestamp, p.deleted,
+                                             u.username, u.user_id, u.profile,
+                                             u.post_count, u.register_date,
                                              p.username.label('guestname')],
         (p.root_post_id == root_post_id) &
         (p.post_id >= post_range_low) &
@@ -598,36 +617,39 @@ def get_flat_view(req, post_id, inc_view_count=True, order='asc'):
         }
         signature = None
         if d['author']['profile'].get('signature'):
-            signature = parse_and_translate(req, d['author']['profile']['signature'],
+            signature = parse_and_render(req, d['author']['profile']['signature'],
                                             signature=True)
         d['author']['signature'] = signature
-        #XXX: this doesn't cache by now
-        d['parsed_text'] = parse_and_translate(req, d['text'])
+
+        # if the post was deleted we remove the text from the return value
+        # so that templates don't display them by chance
+        if d['deleted']:
+            d['text'] = d['parsed_text'] = u''
+        else:
+            d['parsed_text'] = render(req, d['parsed_text'])
         return d
     real_posts = [prepare(num, row) for num, row in inciter(result, page_start)]
 
     # increment view_count
     if inc_view_count:
-        result = ctx.engine.execute(meta.select([p.view_count],
-                                        p.post_id == root_post_id))
-        ctx.engine.execute(posts.update(p.post_id == root_post_id),
-            view_count = result.fetchone()[0] + 1
-        )
+        ctx.engine.execute(posts.update(p.post_id == root_post_id,
+            values={'view_count': p.view_count + 1}
+        ))
 
     # and another query for the overview page
-    result = ctx.engine.execute(meta.select([p.post_id, p.title, p.forum_id,
-                                             p.timestamp, u.user_id, u.username,
-                                             f.name, p.username.label('guestname')],
+    row = ctx.engine.execute(meta.select([p.post_id, p.title, p.forum_id, p.locked,
+                                          p.timestamp, u.user_id, u.username,
+                                          f.name, p.username.label('guestname')],
         (p.post_id == root_post_id) &
         (u.user_id == p.author_id) &
         (f.forum_id == p.forum_id)
-    ))
-    row = result.fetchone()
+    )).fetchone()
     return {
         'post_id':      row['post_id'],
         'url':          ctx.make_url('post', row['post_id']),
         'title':        row['title'],
         'timestamp':    row['timestamp'],
+        'locked':       row['locked'],
         'forum': {
             'forum_id':     row['forum_id'],
             'name':         row['name'],
@@ -653,12 +675,11 @@ def get_last_thread_change(req, post_id):
     no thread with the requested root_post_id exists)
     """
     #XXX: doesn't cover edits
-    result = req.ctx.engine.execute(meta.select([posts.c.timestamp],
-        (posts.c.root_post_id == post_id),
+    row = req.ctx.engine.execute(meta.select([posts.c.timestamp],
+        posts.c.root_post_id == post_id,
         order_by=[meta.desc(posts.c.post_id)],
         limit=1
-    ))
-    row = result.fetchone()
+    )).fetchone()
     if row is None:
         return
     return row[0]
@@ -669,13 +690,8 @@ def get_posts_per_page(req):
     Return the number of posts a user wishes to display on the
     flat view page.
     """
-    try:
-        posts_per_page = req.user.settings['posts_per_page']
-        if posts_per_page is not None:
-            return posts_per_page
-    except KeyError:
-        pass
-    return req.ctx.cfg.get_int('board', 'posts_per_page', 15)
+    return req.user.settings.get('posts_per_page') or \
+           req.ctx.cfg.get_int('board', 'posts_per_page', 15)
 
 
 def get_threads_per_page(req):
@@ -683,13 +699,8 @@ def get_threads_per_page(req):
     Return the number of posts a users whishes to display on the
     viewforum page.
     """
-    try:
-        threads_per_page = req.user.settings['threads_per_page']
-        if threads_per_page is not None:
-            return threads_per_page
-    except KeyError:
-        pass
-    return req.ctx.cfg.get_int('board', 'threads_per_page', 20)
+    return req.user.settings.get('threads_per_page') or \
+           req.ctx.cfg.get_int('board', 'threads_per_page', 20)
 
 
 def get_view_mode(req):
@@ -717,11 +728,11 @@ def quote_post(req, post_id):
     u = users.c
     _ = req.gettext
 
-    result = req.ctx.engine.execute(meta.select([p.title, p.text, u.username],
+    row = req.ctx.engine.execute(meta.select([p.title, p.deleted,
+                                              p.text, u.username],
         (p.post_id == post_id) &
         (u.user_id == p.author_id)
-    ))
-    row = result.fetchone()
+    )).fetchone()
     if row is None:
         # XXX: ValueError?
         raise ValueError('post %s does not exist')
@@ -730,7 +741,10 @@ def quote_post(req, post_id):
     title = row['title']
     if not title.startswith(suffix):
         title = u'%s %s' % (suffix, title)
-    text = quote_text(req, row['text'], row['username'])
+    if row['deleted']:
+        text = u''
+    else:
+        text = quote_text(req, row['text'], row['username'])
     return text, title
 
 
@@ -752,14 +766,70 @@ def edit_post(req, post_id):
     return tuple(row)
 
 
-class _Site(object):
-    """A special singleton representing a whole site."""
+def do_recount(con, forum_id):
+    """
+    Recount a forum. This function is for example used by the `Forum` model.
+    Pass it an connection object (or engine) and the `forum_id` to trigger
+    the recount process. It returns the post and thread count as tuple in
+    the form ``(post_count, thread_count, local_thread_count)``.
 
-    object_id = 0
+    This method also updates the ``local_thread_count`` column which is helds
+    the number of threads in the current forum excluding subforums. It's
+    used for generating the pagination of a forum page.
+    """
+    affected_forums = [forum_id]
+    def find_child_forums(forum_id):
+        for row in con.execute(meta.select([forums.c.forum_id],
+                               forums.c.parent_id == forum_id)):
+            affected_forums.append(row[0])
+            find_child_forums(row[0])
+    find_child_forums(forum_id)
 
-    def __repr__(self):
-        return '<%s>' % self.__class__.__name__
-Site = _Site()
+    post_count = con.execute(meta.select([meta.func.count(posts.c.post_id)],
+        posts.c.forum_id.in_(*affected_forums)
+    )).fetchone()[0]
+    thread_count = con.execute(meta.select([meta.func.count(posts.c.post_id)],
+        (posts.c.forum_id.in_(*affected_forums)) &
+        (posts.c.root_post_id == posts.c.post_id)
+    )).fetchone()[0]
+    local_thread_count = con.execute(meta.select([meta.func.count(posts.c.post_id)],
+        (posts.c.forum_id == forum_id) &
+        (posts.c.root_post_id == posts.c.post_id)
+    )).fetchone()[0]
+    con.execute(forums.update(forums.c.forum_id == forum_id),
+        post_count=post_count,
+        thread_count=thread_count,
+        local_thread_count=local_thread_count
+    )
+    return post_count, thread_count, local_thread_count
+
+
+def sync_last_post_column(con, forum_id):
+    """
+    Sync the `last_post_id` column for the given forum.
+    """
+    # find all subforums of this forum, we look for posts
+    # there too
+    affected_forums = [forum_id]
+    def find_child_forums(forum_id):
+        for row in con.execute(meta.select([forums.c.forum_id],
+                               forums.c.parent_id == forum_id)):
+            affected_forums.append(row[0])
+            find_child_forums(row[0])
+    find_child_forums(forum_id)
+
+    # find the last post
+    row = con.execute(meta.select([posts.c.post_id],
+        (posts.c.forum_id.in_(*affected_forums)) &
+        (posts.c.post_id == posts.c.root_post_id),
+        order_by=[meta.desc(posts.c.timestamp)],
+        limit=1
+    )).fetchone()
+
+    # update the columns
+    con.execute(forums.update(forums.c.forum_id == forum_id),
+        last_post_id=row and row[0] or None
+    )
 
 
 class Forum(DatabaseModel):
@@ -775,6 +845,7 @@ class Forum(DatabaseModel):
     def __init__(self, ctx, forum_id):
         self.ctx = ctx
         self.forum_id = forum_id
+        self._old_parent_id = None
         super(Forum, self).__init__(ctx, forums, 'forum_id')
 
     parent_id = lazy_column('parent_id')
@@ -783,10 +854,13 @@ class Forum(DatabaseModel):
     description = lazy_column('description')
     position = lazy_column('position')
     link = lazy_column('link')
+    last_post_id = lazy_column('last_post_id')
+    post_count = lazy_column('post_count')
+    thread_count = lazy_column('thread_count')
+    local_thread_count = lazy_column('local_thread_count')
 
     @staticmethod
-    def create(ctx, name, description="", parent=None, position=None,
-               link=None):
+    def create(ctx, name, description='', parent=None, position=None, link=None):
         """Create a new forum."""
         if isinstance(parent, Forum):
             parent = parent.forum_id
@@ -801,15 +875,88 @@ class Forum(DatabaseModel):
         )
         return Forum(ctx, result.last_inserted_ids()[0])
 
-    def parent_get(self):
-        return Forum(self.ctx, self.parent_id)
-    def parent_set(self, value):
-        if value is None:
+    def delete(self):
+        """Deletes a forum with all the subfourms and posts in it."""
+        def do(con):
+            """
+            This method is called by the transaction system of the
+            context engine. It deletes a forum with all it's subforums
+            and posts stored inside of itself or one of it's subforums.
+            """
+            parent_forums = set()
+
+            def find_parent_forums(con, forum_id):
+                """find all parent forums and add them to the list"""
+                row = con.execute(meta.select([forums.c.parent_id],
+                                  forums.c.forum_id == forum_id)).fetchone()
+                if row and row[0]:
+                    parent_forums.add(row[0])
+                    find_parent_forums(con, row[0])
+
+            def delete_forum(con, forum_id):
+                """look for subforums and delete them"""
+                for subforum in con.execute(meta.select([forums.c.forum_id],
+                                            forums.c.parent_id == forum_id)):
+                    delete_forum(con, subforum.forum_id)
+                # delete posts
+                con.execute(posts.delete(posts.c.forum_id == forum_id))
+                # delete the forum itself
+                con.execute(forums.delete(forums.c.forum_id == forum_id))
+
+            find_parent_forums(con, self.forum_id)
+            delete_forum(con, self.forum_id)
+
+            # recount all posts and threads in the parent forums
+            for forum_id in parent_forums:
+                do_recount(con, forum_id)
+
+        self.ctx.engine.transaction(do)
+
+    def save(self, con=None):
+        """Updates the post count / thread count of old and new parent
+        forums recursively if the parent was changed."""
+        def do(con):
+            # check if the parent id
+            old_parent = get_initial_column_value(self, 'parent_id', con)
+            new_parent = self.parent_id
+            affected_forums = set([old_parent, new_parent])
+
+            # do plain column updating
+            super(Forum, self).save(con)
+
+            def find_parent_forums(forum_id):
+                """find all parent forums and add them to the list"""
+                for row in con.execute(meta.select([forums.c.forum_id],
+                                       forums.c.parent_id == forum_id)):
+                    affected_forums.add(row[0])
+                    find_parent_forums(row[0])
+
+            # if the parent changed, recount posts and threads
+            # in all affected forums (old parent, new parent)
+            # and update the last post columns
+            if old_parent != new_parent:
+                find_parent_forums(new_parent)
+                find_parent_forums(old_parent)
+                for forum_id in affected_forums:
+                    do_recount(con, forum_id)
+                    sync_last_post_column(con, forum_id)
+
+        self.ctx.engine.transaction(do)
+
+    def move(self, forum):
+        """
+        Move a forum to a new parent forum. Or `None` to make it a category
+        on the top level. You can also use the `parent` property to do so.
+        `value` must be a valid forum, forum_id or `None`.
+        """
+        if forum is None:
             self.parent_id = None
-        if isinstance(value, Forum):
-            self.parent_id = value.forum_id
-    parent = property(parent_get, parent_set)
-    del parent_get, parent_set
+        elif isinstance(forum, Forum):
+            self.parent_id = forum.forum_id
+        elif isinstance(forum, (int, long)):
+            self.parent_id = forum
+
+    parent = property(lambda s: Forum(s.ctx, s.parent_id), move)
 
     def __repr__(self):
         return '<%s %d: %r>' % (
@@ -840,6 +987,7 @@ class Thread(DatabaseModel):
     author_id = lazy_column('author_id')
     title = lazy_column('title')
     text = lazy_column('text')
+    locked = lazy_column('locked')
     timestamp = lazy_column('timestamp')
 
     @staticmethod
@@ -856,39 +1004,81 @@ class Thread(DatabaseModel):
             author = ANONYMOUS_USER_ID
         if timestamp is None:
             timestamp = datetime.utcnow()
+
         def do(con):
+            """
+            This method is used as all in one transaction for creating
+            a new thread. This transaction does the following queries:
+
+            -   add new post to the database
+            -   increment author post count
+            -   lookup all forums that held thread/post count information
+            -   update the thread/post count columns of all affected
+                forums.
+            -   update the root_post_id of the newly created post so
+                that it points to itself.
+            """
+            # insert the new thread into the thread table
             result = con.execute(posts.insert(),
-                forum_id = forum,
-                author_id = author,
-                username = username,
-                title = title,
-                text = text,
-                timestamp = timestamp,
-                post_count = 1,
-                view_count = 0
+                forum_id=forum,
+                author_id=author,
+                username=username,
+                title=title,
+                text=text,
+                parsed_text=parse(ctx, text),
+                timestamp=timestamp,
+                post_count=1,
+                view_count=0,
+                deleted=False,
+                locked=False
             )
             thread_id = result.last_inserted_ids()[-1]
-            # increment forum post and thread count
-            old = meta.select([forums.c.post_count, forums.c.thread_count],
-                               forums.c.forum_id == forum)
-            row = con.execute(old).fetchone()
-            con.execute(forums.update(forums.c.forum_id == forum),
-                post_count = row[0] + 1,
-                thread_count = row[1] + 1,
-                last_post_id = thread_id
+
+            # increment author post count
+            if author > -1:
+                con.execute(users.update(users.c.user_id == author,
+                    values={'post_count': users.c.post_count + 1}
+                ))
+
+            # look up all parent forums and update the `last_post_id`,
+            # `thread_count` and `post_count` columns
+            affected_forums = [forum]
+
+            def find_parent_forums(forum_id):
+                """find all parent forums and add them to the list"""
+                row = con.execute(meta.select([forums.c.parent_id],
+                                  forums.c.forum_id == forum_id)).fetchone()
+                if row and row[0]:
+                    affected_forums.append(row[0])
+                    find_parent_forums(row[0])
+            find_parent_forums(forum)
+
+            # iterate through the list of found forums and update the
+            # columns
+            for forum_id in affected_forums:
+                con.execute(forums.update(forums.c.forum_id == forum_id,
+                    values={
+                        'post_count':           forums.c.post_count + 1,
+                        'thread_count':         forums.c.thread_count + 1,
+                        'local_thread_count':   forums.c.local_thread_count + 1
+                    }),
+                    last_post_id=thread_id
+                )
+
+            # now update the root post id
+            con.execute(posts.update(posts.c.post_id == thread_id),
+                root_post_id=thread_id
             )
             return thread_id
-        thread_id = ctx.engine.transaction(do)
-        # XXX: this feels a bit strange
-        t = Thread(ctx, thread_id)
-        t.root_post_id = t.post_id
-        t.save()
-        return t
+
+        # start the query and return a new thread object for the
+        # requested data.
+        return Thread(ctx, ctx.engine.transaction(do))
 
     @staticmethod
     def by_child(ctx, post_id):
         """
-        Return the thread of a given ``post_id``.
+        Return the thread of a given `post_id`.
         """
         result = ctx.engine.execute(meta.select([posts.c.root_post_id],
             (posts.c.post_id == post_id)
@@ -899,13 +1089,89 @@ class Thread(DatabaseModel):
             raise ValueError('post %s does not exist' % post_id)
         return Thread(ctx, row[0])
 
+    def lock(self):
+        """Lock the thread."""
+        self.locked = True
+
+    def unlock(self):
+        """Unlock the thread."""
+        self.locked = False
+
+    def save(self, con=None):
+        """Update the post count / thread count if the thread was moved."""
+
+        def do(con):
+            # get initial data
+            old_forum = get_initial_column_value(self, 'forum_id', con)
+            new_forum = self.forum_id
+            old_text = get_initial_column_value(self, 'text', con)
+            new_text = self.text
+            affected_forums = set([old_forum, new_forum])
+
+            # reparse text if wanted
+            if old_text != new_text:
+                parsed_text = parse(self.ctx, new_text)
+                set_column_value(self, 'parsed_text', parsed_text)
+
+            # do plain column updateing
+            super(Thread, self).save(con)
+
+            def find_parent_forums(forum_id):
+                """find all parent forums and add them to the list"""
+                row = con.execute(meta.select([forums.c.parent_id],
+                                  forums.c.forum_id == forum_id)).fetchone()
+                if row and row[0]:
+                    affected_forums.add(row[0])
+                    find_parent_forums(row[0])
+
+            # if something changed, find the affected forums
+            # and update columns
+            if old_forum != self.forum_id:
+                find_parent_forums(new_forum)
+                find_parent_forums(old_forum)
+                for forum_id in affected_forums:
+                    do_recount(con, forum_id)
+                    sync_last_post_column(con, forum_id)
+
+        self.ctx.engine.transaction(do)
+
+    def delete(self):
+        """
+        Delete a thread with all of it's subposts.
+        """
+        def do(con):
+            # lookup all forums this thread is a member of
+            affected_forums = set([self.forum_id])
+
+            def find_parent_forums(forum_id):
+                """find all parent forums and add them to the list"""
+                row = con.execute(meta.select([forums.c.parent_id],
+                                  forums.c.forum_id == forum_id)).fetchone()
+                if row and row[0]:
+                    affected_forums.add(row[0])
+                    find_parent_forums(row[0])
+
+            find_parent_forums(self.forum_id)
+
+            # delete thread with all posts
+            con.execute(posts.delete(
+                (posts.c.post_id == self.post_id) |
+                (posts.c.root_post_id == self.post_id)
+            ))
+
+            # recount all affected forums
+            for forum_id in affected_forums:
+                do_recount(con, forum_id)
+                sync_last_post_column(con, forum_id)
+        self.ctx.engine.transaction(do)
+
     def reply(self, post_id, author, title, text, timestamp=None,
               no_processor=False):
         """
-        Reply to post ``post_id`` which is a child of the thread.
+        Reply to post `post_id` which is a child of the thread.
         Return the id of the new post.
 
-        If ``author`` is a string it will be an anonymous posting.
+        If `author` is a string it will be an anonymous posting.
         """
         username = None
         if post_id is None:
@@ -931,35 +1197,35 @@ class Thread(DatabaseModel):
                 raise ValueError('The post either does not exist or does not '
                                  'belong to this thread')
             new_post_id = con.execute(posts.insert(),
-                forum_id = self.forum_id,
-                parent_id = post_id,
-                root_post_id = self.post_id,
-                author_id = author,
-                username = username,
-                title = title,
-                text = text,
+                forum_id=self.forum_id,
+                parent_id=post_id,
+                root_post_id=self.post_id,
+                author_id=author,
+                username=username,
+                title=title,
+                text=text,
+                deleted=False,
+                locked=False,
+                parsed_text=parse(self.ctx, text),
                 timestamp = timestamp
             ).last_inserted_ids()[0]
 
             # increment author post count
             if author > -1:
-                old = meta.select([users.c.post_count], users.c.user_id == author)
-                con.execute(users.update(users.c.user_id == author),
-                    post_count = con.execute(old).fetchone()[0] + 1
-                )
+                con.execute(users.update(users.c.user_id == author,
+                    values={'post_count': users.c.post_count + 1}
+                ))
+
             # increment forum post count and update last_post_id
-            old = meta.select([forums.c.post_count],
-                              forums.c.forum_id == self.forum_id)
-            con.execute(forums.update(forums.c.forum_id == self.forum_id),
-                post_count = con.execute(old).fetchone()[0] + 1,
-                last_post_id = new_post_id
+            con.execute(forums.update(forums.c.forum_id == self.forum_id,
+                values={'post_count': forums.c.post_count + 1}),
+                last_post_id=new_post_id
             )
+
             # increment thread post count
-            old = meta.select([posts.c.post_count],
-                              posts.c.post_id == self.post_id)
-            con.execute(posts.update(posts.c.post_id == self.post_id),
-                post_count = con.execute(old).fetchone()[0] + 1
-            )
+            con.execute(posts.update(posts.c.post_id == self.post_id,
+                values={'post_count': posts.c.post_count + 1}
+            ))
             return new_post_id
         return self.ctx.engine.transaction(do)
 
@@ -977,6 +1243,7 @@ class Thread(DatabaseModel):
             d['title'] = title
         if text is not _missing:
             d['text'] = text
+            d['parsed_text'] = parse(self.ctx, text)
         if timestamp is not _missing:
             d['timestamp'] = timestamp
         if not no_processor and 'title' in d and 'text' in d:
@@ -984,6 +1251,12 @@ class Thread(DatabaseModel):
             d['text'], d['title'] = rv
         self.ctx.engine.execute(posts.update(posts.c.post_id == post_id), **d)
 
+    def delete_reply(self, post_id):
+        """Mark a reply as deleted."""
+        self.ctx.engine.execute(posts.update(posts.c.post_id == post_id),
+            deleted = True
+        )
+
     def has_child(self, post_id):
         """Check if a post_id is a child of this thread."""
         result = self.ctx.engine.execute(meta.select([posts.c.root_post_id],
