@@ -13,18 +13,14 @@ from pocoo.http import Response, HttpRedirect, TemplateResponse, \
      AccessDeniedResponse, PageNotFound
 from pocoo.settings import cfg
 from pocoo.template import PagePublisher, render_template
-from pocoo.application import RequestHandler
+from pocoo.application import Page
 
 from pocoo.utils.mail import Email
 from pocoo.utils.net import make_url_context_external
-from pocoo.utils.form import Form, TextField, TextArea, SelectBox
-from pocoo.utils.validators import isNotEmpty, isSameValue, isEmail, \
-     checkTextLength, isOneLetter, mayEmpty
 from pocoo.utils.json import parse_datetime
+from pocoo.utils.tabbox import TabBox, Tab
 
 from pocoo.pkg.core.remotecall import RemoteCallable, export
-from pocoo.pkg.core.validators import isAvailableUsername, isStrongPassword, \
-     isExistingUsername, isAnonymousUsername
 from pocoo.pkg.core.usersettings import UserSettingsPage
 from pocoo.pkg.core.db import users
 from pocoo.pkg.core.forum import get_forum_index, get_forum, get_post_tree, \
@@ -32,13 +28,16 @@ from pocoo.pkg.core.forum import get_forum_index, get_forum, get_post_tree, \
      get_post, Thread, get_last_posts, get_last_thread_change, get_post_pathbar
 from pocoo.pkg.core.user import User, get_user_list, \
      get_user, get_user_avatar, reset_password
-from pocoo.pkg.core.session import get_active_sessions
-from pocoo.pkg.core.textfmt import parse_and_translate, get_editor
+from pocoo.pkg.core.session import get_active_sessions, get_sessions_by_action
+from pocoo.pkg.core.textfmt import parse_and_render, get_editor
+from pocoo.pkg.core.search import search
+from pocoo.pkg.core.forms import LoginForm, RegisterForm, NewPostForm, \
+     MemberListForm, LostPasswordForm
 
 _ = lambda x: x
 
 
-class IndexPage(RequestHandler, PagePublisher, RemoteCallable):
+class IndexPage(Page, PagePublisher, RemoteCallable):
     page_name = 'index'
     relative_url = ''
     handler_regexes = [u'$']
@@ -50,7 +49,7 @@ class IndexPage(RequestHandler, PagePublisher, RemoteCallable):
         )
 
 
-class ForumPage(RequestHandler):
+class ForumPage(Page):
     handler_regexes = [r'forum/(?P<forum_id>\d+)$']
 
     def handle_request(self, req, forum_id):
@@ -72,61 +71,84 @@ class ForumPage(RequestHandler):
         )
 
 
-class PostPage(RequestHandler, RemoteCallable):
+class PostPage(Page, RemoteCallable):
     handler_regexes = [r'post/(?P<post_id>\d+)$']
 
     def handle_request(self, req, post_id):
+        post_id = int(post_id)
+        if not req.user.acl.can_access_post('READ_POST', post_id):
+            return AccessDeniedResponse()
+
+        rights = {
+            'delete_post': req.user.acl.can_access_post('DELETE_POST', post_id),
+            'lock_topic': req.user.acl.can_access_post('LOCK_TOPIC', post_id),
+            'delete_topic': req.user.acl.can_access_post('DELETE_TOPIC', post_id)
+        }
+
         view = get_view_mode(req)
         if view == 'flat':
-            return self._flat_view(req, int(post_id))
+            return self._flat_view(req, post_id, rights)
         else:
-            return self._threaded_view(req, int(post_id))
+            return self._threaded_view(req, post_id, rights)
 
-    def _flat_view(self, req, post_id):
+    def _flat_view(self, req, post_id, rights):
         topic = get_flat_view(req, post_id)
         if topic is None:
             return PageNotFound()
         return TemplateResponse('viewtopic.html',
             topic=topic,
             pathbar=get_post_pathbar(self.ctx, topic['posts'][0]['post_id']),
-            feed_url=self.ctx.make_url('feeds/thread/%d.xml' % post_id)
+            feed_url=self.ctx.make_url('feeds/thread/%d.xml' % post_id),
+            rights=rights
         )
 
-    def _threaded_view(self, req, post_id):
+    def _threaded_view(self, req, post_id, rights):
         thread = get_post_tree(req, post_id)
         if thread is None:
             return PageNotFound()
         return TemplateResponse('viewthread.html',
             thread=thread,
             pathbar=get_post_pathbar(self.ctx, thread['posts'][0]['post_id']),
-            feed_url=self.ctx.make_url('feeds/thread/%d.xml' % post_id)
+            feed_url=self.ctx.make_url('feeds/thread/%d.xml' % post_id),
+            rights=rights
         )
 
     @export('thread.get_post')
     def _get_post(self, req, post_id):
+        if not req.user.acl.can_access_post('READ_POST', post_id):
+            return AccessDeniedResponse()
         post = get_post(req, post_id)
         if post is None:
             return
+
+        #FIXME: Why isn't this working?
+        # After this, req.user.acl.can_access_post('DELETE_POST', post_id) is still False
+        req.user.acl.set_post_privilege('DELETE_POST', post_id, 2)
+
         return render_template(req, 'partial/post.html', {
-            'post':     post
+            'post':     post,
+            'rights':   {
+                'delete_post': req.user.acl.can_access_post('DELETE_POST', post_id),
+            }
         })
 
     @export('thread.tree_requires_update')
     def _tree_requires_update(self, req, post_id, last_update):
         last_update = parse_datetime(last_update)
         last_thread_change = get_last_thread_change(req, post_id)
-        print last_thread_change, last_update
-        return last_thread_change is not None and\
+        return last_thread_change is not None and \
                last_thread_change > last_update
 
     @export('thread.get_tree')
     def _get_tree(self, req, post_id):
+        if not req.user.acl.can_access_post('READ_POST', post_id):
+            return AccessDeniedResponse()
         return render_template(req, 'partial/tree.html', {
-            'posts':    get_post_tree(req, post_id)['posts']
+            'posts':        get_post_tree(req, post_id)['posts']
         })
 
 
-class LoginPage(RequestHandler, PagePublisher):
+class LoginPage(Page, PagePublisher):
     page_name = 'login'
     relative_url = 'login'
     handler_regexes = ['^login$']
@@ -141,10 +163,7 @@ class LoginPage(RequestHandler, PagePublisher):
         msg = u''
         if req.user.identified:
             return HttpRedirect(next)
-        form = Form(req, self, 'POST',
-            TextField('username', validator=isNotEmpty()),
-            TextField('password', validator=isNotEmpty())
-        )
+        form = LoginForm(req, self, 'POST')
         if req.method == 'POST':
             form.update(req.form, prefix='f_')
             if not form.has_errors:
@@ -168,7 +187,7 @@ class LoginPage(RequestHandler, PagePublisher):
         )
 
 
-class LogoutPage(RequestHandler, PagePublisher):
+class LogoutPage(Page, PagePublisher):
     page_name = 'logout'
     relative_url = 'logout'
     handler_regexes = [r'logout$']
@@ -187,37 +206,33 @@ class LogoutPage(RequestHandler, PagePublisher):
         )
 
 
-class RegisterPage(RequestHandler, PagePublisher):
+class RegisterPage(Page, PagePublisher):
     page_name = 'register'
     relative_url = 'register'
     handler_regexes = ['register$']
 
     def handle_request(self, req):
+        coppa = req.ctx.cfg.get_bool('board', 'enable_coppa', False)
+        activation = req.ctx.cfg.get_bool('board', 'email_verification', False)
+
         if req.user.identified:
             return HttpRedirect('')
         if 'activate' in req.args and 'user' in req.args:
             return self.activation(req)
         _ = req.gettext
-        form = Form(req, self, 'POST',
-            TextField('username', validator=isAvailableUsername()),
-            TextField('password', validator=isStrongPassword()),
-            TextField('password2', validator=isSameValue('password',
-                      _('The passwords don\'t match.'))),
-            TextField('email', validator=isEmail())
-        )
+        form = RegisterForm(req, self, 'POST')
         if req.method == 'POST':
             form.update(req.form, prefix='f_')
             if not form.has_errors:
                 d = form.to_dict()
-                requires_activation = req.ctx.cfg.get_bool('board', 'email_verification')
                 user = User.create(req.ctx,
                     d['username'],
                     d['password'],
                     d['email'],
-                    requires_activation
+                    activation
                 )
-                if requires_activation:
-                    link = self.ctx.make_external_url('register?user=%s&key%s' %
+                if activation:
+                    link = self.ctx.make_external_url('register?user=%s&activate=%s' %
                                                       (user.user_id, user.act_key))
                     txt = render_template(req, 'mails/welcome_verification.txt', {
                         'username':       d['username'],
@@ -230,12 +245,13 @@ class RegisterPage(RequestHandler, PagePublisher):
                                  d['email'], txt)
                     mail.send()
                 return TemplateResponse('messages/register.html',
-                    requires_activation=requires_activation,
+                    requires_activation=activation,
                     username=d['username'],
                     email=d['email']
                 )
         return TemplateResponse('register.html',
-            form=form.generate(prefix='f_')
+            form=form.generate(prefix='f_'),
+            coppa_enabled=coppa
         )
 
     def activation(self, req):
@@ -251,6 +267,7 @@ class RegisterPage(RequestHandler, PagePublisher):
         else:
             if not user.active and req.args['activate'] == key:
                 user.activate()
+                user.save()
                 activated = True
                 username = user.username
         return TemplateResponse('messages/activation.html',
@@ -259,7 +276,7 @@ class RegisterPage(RequestHandler, PagePublisher):
         )
 
 
-class NewPostPage(RequestHandler, RemoteCallable):
+class NewPostPage(Page, RemoteCallable):
     handler_regexes = [
         (r'post/(?P<post_id>\d+)/reply$',
          {'action':'reply'}),
@@ -269,7 +286,16 @@ class NewPostPage(RequestHandler, RemoteCallable):
          {'action':'edit'})
     ]
 
+    @export("newpost.box")
+    def box(self, req, page, *args):
+
+        page = int(page)
+        if page == 1:
+            return render_template(req, 'partial/preview.html',
+                {'text':parse_and_render(req, args[0])})
+
     def handle_request(self, req, action, post_id):
+        post_id = int(post_id)
         try:
             thread = Thread.by_child(self.ctx, post_id)
         except ValueError:
@@ -280,145 +306,173 @@ class NewPostPage(RequestHandler, RemoteCallable):
         # not exist. but since we check for that by calling
         # Thread.by_child this shouldn't happen.
         username = _('anonymous')
+        preview = ""
+
         if action == 'reply':
             mode = 'reply'
-            if not req.user.acl.can_access('REPLY_POST', thread):
+            if not req.user.acl.can_access_post('REPLY_POST', post_id):
                 return AccessDeniedResponse()
             text = u''
-            _, title = quote_post(req, post_id)
+            __, title = quote_post(req, post_id)
         elif action == 'quote':
             mode = 'reply'
-            #XXX: maybe we could use REPLY_POST here too
-            if not req.user.acl.can_access('QUOTE_POST', thread):
+            if not req.user.acl.can_access_post('REPLY_POST', post_id):
                 return AccessDeniedResponse()
             text, title = quote_post(req, post_id)
         else:
             mode = 'edit'
-            if not req.user.acl.can_access('EDIT_POST', thread):
+            #XXX: also check for EDIT_OWN_POST
+            if not req.user.acl.can_access_post('EDIT_POST', post_id):
                 return AccessDeniedResponse()
             text, title, username = edit_post(req, post_id)
 
-        form = Form(req, req.environ['APPLICATION_REQUEST'], 'POST',
-            TextField('username', validator=isAnonymousUsername(),
-                      default=username),
-            TextField('title', validator=checkTextLength(3, 60),
-                      default=title),
-            TextArea('text', validator=checkTextLength(3, 10000),
-                     default=text)
-        )
+        form = NewPostForm(req, req.environ['APPLICATION_REQUEST'], 'POST', {
+            'username': username,
+            'title':    title,
+            'text':     text
+        })
+
         if req.method == 'POST':
             form.update(req.form, prefix='f_')
-            if not form.has_errors:
+            if not form.has_errors and 'partial' not in req.form:
                 d = form.to_dict()
-                if req.user.identified:
-                    author = req.user.user_id
+                if "preview" in req.form:
+                    #XXX
+                    req.form['partial'] = 2
                 else:
-                    author = d['username']
-                if action in ('quote', 'reply'):
-                    new_post_id = thread.reply(
-                        post_id = int(post_id),
-                        author = author,
-                        title = d['title'],
-                        text = d['text']
+                    if req.user.identified:
+                        author = req.user.user_id
+                    else:
+                        author = d['username']
+                    if action in ('quote', 'reply'):
+                        new_post_id = thread.reply(
+                            post_id = int(post_id),
+                            author = author,
+                            title = d['title'],
+                            text = d['text']
+                        )
+                    elif action == 'edit':
+                        new_post_id = int(post_id)
+                        thread.edit_reply(
+                            post_id = new_post_id,
+                            author = author,
+                            title = d['title'],
+                            text = d['text']
+                        )
+                    return TemplateResponse('messages/post.html',
+                        mode = mode,
+                        post = {
+                            'id':     new_post_id,
+                            'title':  d['title'],
+                            'url':    self.ctx.make_url('post', new_post_id),
+                        }
                     )
-                elif action == 'edit':
-                    new_post_id = int(post_id)
-                    thread.edit_reply(
-                        post_id = new_post_id,
-                        author = author,
-                        title = d['title'],
-                        text = d['text']
-                    )
-                return TemplateResponse('messages/post.html',
-                    mode = mode,
-                    post = {
-                        'id':     new_post_id,
-                        'title':  d['title'],
-                        'url':    self.ctx.make_url('post', new_post_id),
-                    }
-                )
+
+        f = form.generate(prefix='f_')
+
+        box = TabBox(req, self.box, True,
+            Tab(_("Latest Posts"), lazy=True, short_name="lposts"),
+            Tab(_("Preview"), static=False, send=[f["text"]["name"]],
+                short_name="preview")
+        )
+
         js, options = get_editor(req)
         return TemplateResponse('newpost.html',
             mode = mode,
-            form = form.generate(prefix='f_'),
+            form = f,
             show_username_entry = not req.user.identified,
             editor_options = options,
             editor_javascript = js,
             post_id = thread.root_post_id,
-            pathbar=get_post_pathbar(req.ctx, post_id)
+            pathbar=get_post_pathbar(req.ctx, post_id),
+            preview = preview,
+            tabbox = box
         )
 
-    @export("newpost.preview")
-    def preview(self, req, text):
-        return parse_and_translate(req, text)
-
-    @export("newpost.get_latest_posts")
-    def get_latest_posts(self, req, root_post_id):
-        post_list = get_last_posts(req, root_post_id, 15)
-        return {
-            'html': render_template(req, 'latestposts.html', {'posts': post_list}),
-            'last_post': post_list[0]["post_id"]
-        }
-
-    @export("newpost.check_new_posts")
-    def check_new_posts(self, req, root_post_id, latest_post):
-        post = get_last_posts(req, root_post_id, 1)[0]
-        if latest_post != post["post_id"]:
-            post_list = []
-            z = 1
-            while post["post_id"] != latest_post:
-                post_list.append(post)
-                post = get_last_posts(req, root_post_id, 1, z)[0]
-                z += 1
-            return {
-                'html': render_template(req, 'latestposts.html', {'posts': post_list}),
-                'last_post': post_list[0]["post_id"]
-            }
+    @export('newpost.check')
+    def check(self, req, root_post_id, latest_post):
+        latest_post = latest_post and int(latest_post) or None
+        post_list = []
+        if not latest_post:
+            post_list = get_last_posts(req, root_post_id, 5)
         else:
-            return False
+            post = get_last_posts(req, root_post_id, 1)[0]
+            if latest_post != post['post_id']:
+                z = 1
+                while post['post_id'] != latest_post and z < 5:
+                    post_list.append(post)
+                    post = get_last_posts(req, root_post_id, 1, z)[0]
+                    z += 1
+
+        return post_list and {
+                'html': render_template(req, 'partial/latestposts.html',
+                    {'posts': post_list }),
+                'latestpost': post_list[0]["post_id"]
+            } or ""
 
 
-class NewThreadPage(RequestHandler):
+class NewThreadPage(Page, RemoteCallable):
     handler_regexes = [r'forum/(?P<forum_id>\d+)/new$']
 
+    @export("newthread.box")
+    def box(self, req, page, *args):
+        page = int(page)
+        if page == 0:
+            return render_template(req, 'partial/preview.html',
+                {'text':parse_and_render(req, len(args) and args[0] or "")})
+
     def handle_request(self, req, forum_id):
-        # TODO: Check whether user is allowed to do this
         _ = req.gettext
+        preview = ""
         forum_id = int(forum_id)
-        form = Form(req, req.environ['APPLICATION_REQUEST'], 'POST',
-            TextField('username', validator=isAnonymousUsername(),
-                      default=_('anonymous')),
-            TextField('title', validator=checkTextLength(3, 60)),
-            TextArea('text', validator=checkTextLength(3, 10000))
-        )
+        if not req.user.acl.can_access_forum('CREATE_THREAD', forum_id):
+            return AccessDeniedResponse()
+
+        form = NewPostForm(req, req.environ['APPLICATION_REQUEST'], 'POST', {
+            'username': _('anonymous')
+        })
+
         if req.method == 'POST':
             form.update(req.form, prefix='f_')
-            if not form.has_errors:
+            if not form.has_errors and 'partial' not in req.form:
                 d = form.to_dict()
-                if req.user.identified:
-                    author = req.user.user_id
-                else:
-                    author = d['username']
-                thread = Thread.create(req.ctx, forum_id, author,
-                                       d['title'], d['text'])
-                return TemplateResponse('messages/thread.html',
-                    thread = {
-                        'title': thread.title,
-                        'url':   req.ctx.make_url('post', thread.root_post_id),
-                    }
-                )
+                if 'preview' in req.form:
+                    #XXX
+                    req.form['partial'] = 3
+                elif 'submit' in req.form:
+                    if req.user.identified:
+                        author = req.user.user_id
+                    else:
+                        author = d['username']
+                    thread = Thread.create(req.ctx, forum_id, author,
+                                        d['title'], d['text'])
+                    return TemplateResponse('messages/thread.html',
+                        thread = {
+                            'title': thread.title,
+                            'url':   req.ctx.make_url('post', thread.root_post_id),
+                        }
+                    )
+
+        f = form.generate(prefix='f_')
+
+        box = TabBox(req, self.box, True,
+            Tab(_("Preview"), static=False, send=[f["text"]["name"]],
+                short_name="preview")
+        )
 
         js, options = get_editor(req)
         return TemplateResponse('newthread.html',
-            form = form.generate(prefix='f_'),
             show_username_entry = not req.user.identified,
+            form = f,
             editor_options = options,
             editor_javascript = js,
-            pathbar = get_forum_pathbar(self.ctx, forum_id)
+            pathbar = get_forum_pathbar(self.ctx, forum_id),
+            preview = preview,
+            tabbox = box
         )
 
 
-class MemberListPage(RequestHandler, PagePublisher):
+class MemberListPage(Page, PagePublisher):
     page_name = 'memberlist'
     relative_url = 'users/'
     handler_regexes = [r'users/$']
@@ -427,19 +481,10 @@ class MemberListPage(RequestHandler, PagePublisher):
         _ = req.gettext
         generate = False
 
-        form = Form(req, self, 'GET',
-            TextField('letter', validator=mayEmpty(isOneLetter())),
-            SelectBox('order_by', [
-                ('user_id',         _('User ID')),
-                ('username',        _('Username')),
-                ('register_date',   _('Register date')),
-                ('post_count',      _('Number of Posts'))
-            ], default='user_id'),
-            SelectBox('direction', [
-                ('desc',        _('Descending')),
-                ('asc',         _('Ascending'))
-            ], default='asc')
-        )
+        form = MemberListForm(req, self, 'GET', {
+            'order_by':         'user_id',
+            'direction':        'asc'
+        })
 
         if 'letter' in req.args or 'order_by' in req.args \
            or 'direction' in req.args:
@@ -451,7 +496,7 @@ class MemberListPage(RequestHandler, PagePublisher):
         if generate:
             d = form.to_dict()
             userlist = get_user_list(self.ctx,
-                order_by=getattr(users.c, d['order_by']),
+                order_by=d['order_by'],
                 order=d['direction'],
                 letter=d['letter'] or None,
                 hide_internal=True
@@ -465,7 +510,7 @@ class MemberListPage(RequestHandler, PagePublisher):
         )
 
 
-class UserPage(RequestHandler):
+class UserPage(Page):
     handler_regexes = [r'users/(?P<username>.+)$']
 
     def handle_request(self, req, username):
@@ -479,7 +524,7 @@ class UserPage(RequestHandler):
             resp['Content-Type'] = 'image/png'
             return resp
         # display user
-        user = get_user(req, username)
+        user = get_user(req.ctx, username)
         if user is None:
             return PageNotFound()
         return TemplateResponse('user.html',
@@ -487,94 +532,25 @@ class UserPage(RequestHandler):
         )
 
 
-class GotoPage(RequestHandler):
-    handler_regexes = ['goto$']
+class SearchPage(Page, PagePublisher):
+    page_name = 'search'
+    relative_url = 'search/'
+    handler_regexes = [r'search/$']
 
     def handle_request(self, req):
-        if 'post' in req.args:
-            post_id = req.args['post']
-            post = get_post(req.ctx, post_id)
-            while post.parent: #get root_post
-                post = post.parent or post
-            thread = req.db.get(Thread, Thread.c.root_post_id == post.post_id)
-            return HttpRedirect(thread.url)
-        else:
-            return HttpRedirect(self.ctx.make_external_url('/'))
+        if 'q' in req.values:
+            try:
+                page = abs(int(req.values['page']))
+            except (ValueError, KeyError):
+                page = 1
+            rv = search(req, req.values['q'], 30, page)
+            if rv is None:
+                return PageNotFound()
+            return TemplateResponse('search/results.html', **rv)
+        return TemplateResponse('search/form.html')
 
 
-class SplitPage(RequestHandler, RemoteCallable):
-    handler_regexes = [r'post/(?P<post_id>\d+)/split$']
-
-    def handle_request(self, req, post_id):
-        try:
-            thread = Thread.by_child(req.ctx, post_id)
-        except ValueError:
-            return PageNotFound()
-        if not req.user.acl.can_access('MODERATOR', thread.forum_id):
-            return AccessDeniedResponse()
-        return TemplateResponse('split.html',
-            thread = {'root_post_id':thread.root_post_id, 'title':thread.title}
-        )
-
-    @export('split.get_post_tree')
-    def split(self, req, root_post_id):
-        def iterate_items(post_list):
-            tmp = []
-            for post in post_list:
-                tmp.append({
-                    'post_id': post["post_id"],
-                    'title': post["title"],
-                    'children': "children" in post and \
-                                iterate_items(post["children"]) or [],
-                    'parent_id': post["parent_id"]
-                })
-            return tmp
-
-        post_tree = get_post_tree(req, root_post_id)
-
-        if not req.user.acl.can_access('MODERATOR', post_tree["forum"]["forum_id"]):
-            return AccessDeniedResponse()
-
-        return {
-            'posts':iterate_items(post_tree["posts"]),
-            'thread':{
-                'forum_id':post_tree["forum"]["forum_id"],
-                'root_post_id':post_tree["post_id"]
-            }
-        }
-
-    @export('split.commit')
-    def commit(self, req, changes):
-        # TODO: Check whether operation is allowed at every subforum of a thread
-        #       the user changes
-        for tmp in changes.iteritems():
-            change = tmp[1]
-            id = change['id']
-            new_pid = change['new_pid']
-            old_pid = change['old_pid']
-            act_tid = change['act_tid']
-            old_tid = change['old_tid']
-            post = req.db.get(Post, Post.c.post_id == id)
-            if new_pid != -1:
-                post.parent_id = new_pid
-            else:
-                if act_tid == -1:
-                    #XXX: forum_id
-                    thread = Thread(
-                        name = post.name,
-                        forum_id = 1,
-                        root_post_id = post.post_id
-                    )
-                    req.db.save(thread)
-                else:
-                    thread = req.db.get(Thread, Thread.c.thread_id == act_tid)
-                    thread.name = post.name
-                post.parent_id = thread.thread_id
-        req.db.flush()
-        return ""
-
-
-class WhoIsOnlinePage(RequestHandler, PagePublisher):
+class WhoIsOnlinePage(Page, PagePublisher):
     page_name = 'whoisonline'
     relative_url = 'whoisonline'
     handler_regexes = [r'whoisonline$']
@@ -589,7 +565,7 @@ class WhoIsOnlinePage(RequestHandler, PagePublisher):
         )
 
 
-class UserSettings(RequestHandler, PagePublisher):
+class UserSettings(Page, PagePublisher):
     page_name = 'settings'
     relative_url = 'settings/'
     handler_regexes = [
@@ -636,7 +612,7 @@ class UserSettings(RequestHandler, PagePublisher):
         return TemplateResponse(template, **context)
 
 
-class LostPasswordPage(RequestHandler, PagePublisher):
+class LostPasswordPage(Page, PagePublisher):
     page_name = 'lostpassword'
     relative_url = 'lostpassword'
     handler_regexes = ['lostpassword$']
@@ -649,10 +625,7 @@ class LostPasswordPage(RequestHandler, PagePublisher):
         _ = req.gettext
         ctx = self.ctx
         msg = None
-        form = Form(req, self, 'POST',
-            TextField('username', validator=isExistingUsername()),
-            TextField('email', validator=isEmail())
-        )
+        form = LostPasswordForm(req, self, 'POST')
 
         if req.method == 'POST':
             form.update(req.form, prefix='f_')
@@ -675,3 +648,131 @@ class LostPasswordPage(RequestHandler, PagePublisher):
             form = form.generate(prefix='f_'),
             msg = msg
         )
+
+
+class PostOperationsPage(Page):
+    handler_regexes = [
+        (r'post/(?P<post_id>\d+)/lock$',
+         {'action':'lock'}),
+        (r'post/(?P<post_id>\d+)/unlock$',
+         {'action':'unlock'}),
+        (r'post/(?P<post_id>\d+)/delete$',
+         {'action':'delete_post'}),
+        (r'post/(?P<post_id>\d+)/delete_thread$',
+         {'action':'delete_thread'})
+    ]
+
+    def handle_request(self, req, action, post_id):
+        """
+        Use the following command for testing:
+            req.user.acl.set_forum_privilege('MODERATE_FORUM', forum_id, 2)
+        Execute the following line also if you want to store the permission in the
+        database:
+            req.user.save()
+        """
+        try:
+            thread = Thread.by_child(req.ctx, post_id)
+        except ValueError:
+            return PageNotFound()
+
+        if not req.user.acl.can_access_forum('MODERATE_FORUM', thread.forum_id):
+            return AccessDeniedResponse()
+
+        if req.method == 'POST' and 'continue' in req.form:
+            if action == 'delete_post':
+                thread.delete_reply(post_id)
+            elif action == 'lock':
+                thread.lock()
+            elif action == 'unlock':
+                thread.unlock()
+            elif action == 'delete_thread':
+                thread.delete()
+            thread.save()
+            return TemplateResponse('messages/operation_done.html',
+                action = action,
+                url = self.ctx.make_url('post/%s' % post_id)
+            )
+        elif 'abort' in req.form:
+            return HttpRedirect(self.ctx.make_url('post/%s' % post_id))
+        else:
+            return TemplateResponse('messages/operation_request.html',
+                action = action
+            )
+
+"""
+Old SplitPage
+
+class SplitPage(Page, RemoteCallable):
+    handler_regexes = [r'post/(?P<post_id>\d+)/split$']
+
+    def handle_request(self, req, post_id):
+        try:
+            thread = Thread.by_child(req.ctx, post_id)
+        except ValueError:
+            return PageNotFound()
+        if not req.user.acl.can_access('MODERATOR', thread.forum_id):
+            return AccessDeniedResponse()
+        return TemplateResponse('split.html',
+            thread = {
+                'root_post_id': thread.root_post_id,
+                'title':        thread.title
+            }
+        )
+
+    @export('split.get_post_tree')
+    def split(self, req, root_post_id):
+        def iterate_items(post_list):
+            tmp = []
+            for post in post_list:
+                tmp.append({
+                    'post_id':  post['post_id'],
+                    'title':    post['title'],
+                    'children': 'children' in post and \
+                                iterate_items(post['children']) or [],
+                    'parent_id': post['parent_id']
+                })
+            return tmp
+
+        post_tree = get_post_tree(req, root_post_id)
+
+        if not req.user.acl.can_access('MODERATOR', post_tree['forum']['forum_id']):
+            return AccessDeniedResponse()
+
+        return {
+            'posts':    iterate_items(post_tree["posts"]),
+            'thread': {
+                'forum_id':     post_tree["forum"]["forum_id"],
+                'root_post_id': post_tree["post_id"]
+            }
+        }
+
+    @export('split.commit')
+    def commit(self, req, changes):
+        # TODO: Check whether operation is allowed at every subforum of a thread
+        #       the user changes
+        for tmp in changes.iteritems():
+            change = tmp[1]
+            id = change['id']
+            new_pid = change['new_pid']
+            old_pid = change['old_pid']
+            act_tid = change['act_tid']
+            old_tid = change['old_tid']
+            post = req.db.get(Post, Post.c.post_id == id)
+            if new_pid != -1:
+                post.parent_id = new_pid
+            else:
+                if act_tid == -1:
+                    #XXX: forum_id
+                    thread = Thread(
+                        name = post.name,
+                        forum_id = 1,
+                        root_post_id = post.post_id
+                    )
+                    req.db.save(thread)
+                else:
+                    thread = req.db.get(Thread, Thread.c.thread_id == act_tid)
+                    thread.name = post.name
+                post.parent_id = thread.thread_id
+        req.db.flush()
+        return ""
+"""
