@@ -1,5 +1,5 @@
 /*
-Last Modified: 21/08/06 21:13:32
+Last Modified: 21/10/06 01:20:22
 
   AJS JavaScript library
     A very small library with a lot of functionality.
@@ -11,11 +11,11 @@ Last Modified: 21/08/06 21:13:32
     Copyright (c) 2005 Bob Ippolito. All rights reserved.
     http://www.opensource.org/licenses/mit-license.php
   VERSION
-    3.0
+    3.20
   SITE
     http://orangoo.com/AmiNation/AJS
 **/
-
+if(!AJS) {
 var AJS = {
   BASE_URL: "",
 
@@ -112,7 +112,7 @@ var AJS = {
     else
       return null;
   },
-  
+
   getLast: function(list) {
     if(list.length > 0)
       return list[list.length-1];
@@ -151,7 +151,7 @@ var AJS = {
     if(end_index)
        l = end_index;
     for(i; i < l; i++)
-      fn.apply(null, [list[i]]);
+      fn.apply(null, [list[i], i]);
   },
 
   filter: function(list, fn, /*optional*/ start_index, end_index) {
@@ -165,7 +165,7 @@ var AJS = {
 
   partial: function(fn) {
     var args = AJS.forceArray(arguments);
-    return AJS.$b(fn, null, args.slice(1, args.length), false, true);
+    return AJS.$b(fn, null, args.slice(1, args.length).reverse(), false, true);
   },
 
 
@@ -212,6 +212,29 @@ var AJS = {
     return class_elements;
   },
 
+  _nodeWalk: function(elm, tag_name, class_name, fn_next_elm) {
+    var p = fn_next_elm(elm);
+    while(p) {
+      if((AJS.nodeName(p) == tag_name || !tag_name) && (p.className == class_name || !class_name))
+        return p;
+      else
+        p = fn_next_elm(p);
+    }
+    return null;
+  },
+
+  getParentBytc: function(elm, tag_name, class_name) {
+    return AJS._nodeWalk(elm, tag_name, class_name, function(m) { return m.parentNode; });
+  },
+
+  getPreviousSiblingBytc: function(elm, tag_name, class_name) {
+    return AJS._nodeWalk(elm, tag_name, class_name, function(m) { return m.previousSibling; });
+  },
+
+  getNextSiblingBytc: function(elm, tag_name, class_name) {
+    return AJS._nodeWalk(elm, tag_name, class_name, function(m) { return m.nextSibling; });
+  },
+
   //Shortcut: AJS.$f
   getFormElement: function(form, name) {
     form = AJS.$(form);
@@ -223,6 +246,21 @@ var AJS = {
     return r;
   },
 
+  formContents: function(form) {
+    form = AJS.$(form);
+    var r = {};
+    var find_and_set = function(elm_name) {
+      var children = AJS.$bytc(elm_name, null, form);
+      AJS.map(children, function(c) {
+        if(c.name)
+          r[c.name] = c.value || '';
+      });
+    };
+    find_and_set('input');
+    find_and_set('textarea');
+    return r;
+  },
+
   getBody: function() {
     return AJS.$bytc('body')[0]
   },
@@ -271,7 +309,7 @@ var AJS = {
   //Shortcut: AJS.RCN
   replaceChildNodes: function(elm/*, elms...*/) {
     var child;
-    while ((child = elm.firstChild)) 
+    while ((child = elm.firstChild))
       elm.removeChild(child);
     if (arguments.length < 2)
       return elm;
@@ -368,7 +406,6 @@ var AJS = {
     AJS.TN = function(text) { return document.createTextNode(text) };
   },
 
-
   getCssDim: function(dim) {
     if(AJS.isString(dim))
       return dim;
@@ -378,28 +415,45 @@ var AJS = {
 
   setWidth: function(/*elm1, elm2..., width*/) {
     var args = AJS.flattenList(arguments);
-    var w = AJS.getLast(args);
-    AJS.map(args, function(elm) { elm.style.width = AJS.getCssDim(w)}, 0, args.length-1);
+    var w = args.pop();
+    AJS.map(args, function(elm) { elm.style.width = AJS.getCssDim(w)});
   },
   setHeight: function(/*elm1, elm2..., height*/) {
     var args = AJS.flattenList(arguments);
-    var h = AJS.getLast(args);
-    AJS.map(args, function(elm) { elm.style.height = AJS.getCssDim(h)}, 0, args.length-1);
+    var h = args.pop();
+    AJS.map(args, function(elm) { elm.style.height = AJS.getCssDim(h)});
   },
   setLeft: function(/*elm1, elm2..., left*/) {
     var args = AJS.flattenList(arguments);
-    var l = AJS.getLast(args);
-    AJS.map(args, function(elm) { elm.style.left = AJS.getCssDim(l)}, 0, args.length-1);
+    var l = args.pop();
+    AJS.map(args, function(elm) { elm.style.left = AJS.getCssDim(l)});
   },
   setTop: function(/*elm1, elm2..., top*/) {
     var args = AJS.flattenList(arguments);
-    var t = AJS.getLast(args);
-    AJS.map(args, function(elm) { elm.style.top = AJS.getCssDim(t)}, 0, args.length-1);
+    var t = args.pop();
+    AJS.map(args, function(elm) { elm.style.top = AJS.getCssDim(t)});
   },
   setClass: function(/*elm1, elm2..., className*/) {
     var args = AJS.flattenList(arguments);
-    var t = AJS.getLast(args);
-    AJS.map(args, function(elm) { elm.className = t}, 0, args.length-1);
+    var c = args.pop();
+    AJS.map(args, function(elm) { elm.className = c});
+  },
+  addClass: function(/*elm1, elm2..., className*/) {
+    var args = AJS.flattenList(arguments);
+    var cls = args.pop();
+    var add_class = function(o) {
+      if(!new RegExp("(^|\\s)" + cls + "(\\s|$)").test(o.className))
+        o.className += (o.className ? " " : "") + cls;
+    };
+    AJS.map(args, function(elm) { add_class(elm); });
+  },
+  removeClass: function(/*elm1, elm2..., className*/) {
+    var args = AJS.flattenList(arguments);
+    var cls = args.pop();
+    var rm_class = function(o) {
+      o.className = o.className.replace(new RegExp("\\s?" + cls), "");
+    };
+    AJS.map(args, function(elm) { rm_class(elm); });
   },
 
   setHTML: function(elm, html) {
@@ -462,15 +516,24 @@ var AJS = {
 
   getRequest: function(url, data, type) {
     //Extend the privlege so we can make cross host reqs
-    try { 
-      netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead"); 
+    try {
+      netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
     } catch (e) { }
 
     if(!type)
       type = "POST";
     var req = AJS.getXMLHttpRequest();
-    if(url.indexOf("http://") == -1)
-      url = AJS.BASE_URL + url;
+
+    if(url.indexOf("http://") == -1) {
+      if(AJS.BASE_URL != '') {
+        if(AJS.BASE_URL.lastIndexOf('/') != AJS.BASE_URL.length-1)
+          AJS.BASE_URL += '/';
+        url = AJS.BASE_URL + url;
+      }
+      else
+        url = window.location + url;
+    }
+
     req.open(type, url, true);
     if(type == "POST")
       req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
@@ -492,6 +555,7 @@ var AJS = {
         }
         else if(status == 500) {
           alert(req.responseText);
+          d.errback();
         }
         else {
           d.errback();
@@ -626,15 +690,15 @@ var AJS = {
   absolutePosition: function(elm) {
     var posObj = {'x': elm.offsetLeft, 'y': elm.offsetTop};
     if(elm.offsetParent) {
-      var temp_pos =	AJS.absolutePosition(elm.offsetParent);
+      var temp_pos =    AJS.absolutePosition(elm.offsetParent);
       posObj.x += temp_pos.x;
       posObj.y += temp_pos.y;
     }
     // safari bug
-    if (AJS.isSafari() && elm.style.position == 'absolute' ) { 
+    if (AJS.isSafari() && elm.style.position == 'absolute' ) {
       posObj.x -= document.body.offsetLeft;
       posObj.y -= document.body.offsetTop;
-    } 
+    }
     return posObj;
   },
 
@@ -788,11 +852,15 @@ var AJS = {
     window.setTimeout(fn_no_send, interval);
   },
 
+  preventDefault: function(e) {
+    if(AJS.isIe()) window.event.returnValue = false;
+    else e.preventDefault();
+  },
+
 
 ////
 // Drag and drop
 ////
-
   dragAble: function(elm, /*optional*/ handler, args) {
     if(!args)
       args = {};
@@ -843,12 +911,9 @@ var AJS = {
     AJS.AEV(document, 'mousemove', AJS._dragMove, false, true);
     AJS.AEV(document, 'mouseup', AJS._dragStop, false, true);
 
-    if (AJS.isIe()) {
+    if (AJS.isIe())
       window.event.cancelBubble = true;
-      window.event.returnValue = false;
-    }
-    else
-      e.preventDefault();
+    AJS.preventDefault(e);
   },
 
   _initDragElm: function(elm) {
@@ -1029,7 +1094,7 @@ AJSDeferred = function(req) {
   this.callbacks = [];
   this.errbacks = [];
   this.req = req;
-};
+}
 AJSDeferred.prototype = {
   excCallbackSeq: function(req, list) {
     var data = req.responseText;
@@ -1080,8 +1145,11 @@ AJSDeferred.prototype = {
       this.req.send("");
     }
   }
-}
+};
 
 //Prevent memory-leaks
 AJS.addEventListener(window, 'unload', AJS._unloadListeners);
 AJS._createDomShortcuts()
+}
+
+script_loaded = true;
