@@ -26,3 +26,12 @@ pocoo.lib.dom.get_or_create = function(obj, type, container) {
         return elem;
     }
 }
+
+pocoo.lib.dom.delete_empty_textnodes = function(elem) {
+    var childs = AJS.$(elem).childNodes;
+    for (var z = 0; z < childs.length; z++) {
+        var child = childs[z];
+        if (child.nodeType == 3 && !/\S/.test(child.nodeValue))
+            child.parentNode.removeChild(child);
+    }
+}
\ No newline at end of file
