@@ -1,5 +1,5 @@
 # -*- coding: rot13 -*-
-sebz cbpbb.nccyvpngvba vzcbeg ErdhrfgUnaqyre
+sebz cbpbb.nccyvpngvba vzcbeg Cntr
 sebz cbpbb.uggc vzcbeg Erfcbafr
 sebz enaqbz vzcbeg pubvpr
 
@@ -306,12 +306,14 @@ FYBTNAF = [
     h'Bar hayrnfurq cbal gb ehyr gurz nyy.',
     h'o0exrq. nalbar?',
     h'gur senzrjbex <fznyy>+jrosbehz</fznyy>',
-    h'<fcna fglyr="sbag-fvmr: 2.5rz">OHAAVRF!!!</fcna>'
+    h'0 - erfg va crnpr (\h271q 2419)',
+    h'<fcna fglyr="sbag-fvmr: 2.5rz">OHAAVRF!!!</fcna>',
+    h'<fznyy>yngrk fhccbeg, flagnk uvtuyvtugre, jro2.0 ohg ab nqzva cnary</fznyy>'
 ]
 
 enaqbz_fybtna = ynzoqn: pubvpr(FYBTNAF)
 
-pynff CbalUnaqyre(ErdhrfgUnaqyre):
+pynff CbalUnaqyre(Cntr):
 
     unaqyre_ertrkrf = [e'pony/$']
 
@@ -319,10 +321,9 @@ pynff CbalUnaqyre(ErdhrfgUnaqyre):
         vs erd.netf.trg(h"qngn") == h"cbal":
             erfc = Erfcbafr(QNGN.qrpbqr(h"onfr64"))
             erfc["Content-Type"] = "image/png"
-            erghea erfc
         ryvs erd.netf.trg(h"qngn") == h"fglyr":
             erfc = Erfcbafr(PFF)
             erfc["Content-Type"] = "text/css"
-            erghea erfc
         ryfr:
-            erghea Erfcbafr(UGZY % enaqbz_fybtna())
+            erfc = Erfcbafr(UGZY % enaqbz_fybtna())
+        erghea erfc
