@@ -399,8 +399,6 @@ class Configuration(object):
                 raise MissingConfigValue('mandatory value %s.%s not set' %
                                          (section, key))
 
-# TODO: add generic get_type() function.
-
     def get_bool(self, section, key, default=missing):
         """
         Get the value, and convert to boolean.
@@ -436,6 +434,26 @@ class Configuration(object):
                                      'a float but cannot be converted '
                                      'to one' % (section, key))
 
+    def get_list(self, section, key, default=missing):
+        """
+        Get the value, and convert to list.
+        """
+        rv = self.get(section, key, default)
+        if not isinstance(rv, list):
+            raise MissingConfigValue('config value %s.%s should be '
+                                     'a list' % (section, key))
+        return rv
+
+    def get_dict(self, section, key, default=missing):
+        """
+        Get the value, and convert to list.
+        """
+        rv = self.get(section, key, default)
+        if not isinstance(rv, dict):
+            raise MissingConfigValue('config value %s.%s should be '
+                                     'a dict' % (section, key))
+        return rv
+
     def set(self, section, key, value):
         """
         Set ``section.key`` to ``value``.
@@ -443,8 +461,6 @@ class Configuration(object):
         # convert to string for ini representation
         if isinstance(value, bool):
             value = value and 'true' or 'false'
-        else:
-            value = unicode(value)
 
         node = self._data
         for bit in section.split('.'):
@@ -452,6 +468,9 @@ class Configuration(object):
                 node[bit] = {'__parent__': node}
             node = node[bit]
 
+        # FIXME: There are quite a few problems here, especially new
+        #        sections are not written to the config file.
+
         if key not in node:
             # for raw writing
             self._new_keys.setdefault(section, []).append(key)
